package main

import (
	"crypto/sha256"
	"database/sql"

	"bitbucket.org/prmunty/middleware/auth"
	migration "bitbucket.org/prmunty/util/dbmigration/db"
	sqlite "bitbucket.org/prmunty/util/dbmigration/sqliteutils"
	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 3.0
	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			AddNewHiddenUsers,
			AddLocalHostConfigTables,
		)
	})

}

func AddNewHiddenUsers(tx *sql.Tx) (err error) {
	return migration.DoUntilErr(tx,
		func(tx *sql.Tx) error { return addNewUser(tx, "Frick Service", hashPin("37378423"), auth.FrickService) },
		func(tx *sql.Tx) error { return addNewUser(tx, "Ringout", hashPin("7464688"), auth.Ringout) },
		func(tx *sql.Tx) error { return addNewUser(tx, "Controls", hashPin("26687657"), auth.Factory) },
	)

}

func addNewUser(tx *sql.Tx, firstName, pin string, secLevel auth.SecLevel) error {
	if _, err := tx.Exec("DELETE FROM users where secLevel = ?", secLevel); err != nil && err != sql.ErrNoRows {
		return err
	}

	_, err := tx.Exec(`INSERT INTO users (firstName, lastname, email, phone, carrier, pin,secLevel) 
             VALUES (?, "User", '', '','', ?, ?)`, firstName, pin, int(secLevel))
	return err

}

// hashPin hashes a Pin and returns the hashed value as a string
func hashPin(pin string) string {
	bytesArr := sha256.Sum256([]byte(pin))
	hashString := string(bytesArr[0:])
	return hashString
}

func AddLocalHostConfigTables(tx *sql.Tx) error {

	return migration.DoUntilErr(tx,
		addLocalHostConfig,
		addLocalHostComponents,
	)

}

func addLocalHostConfig(tx *sql.Tx) (err error) {
	if !(sqlite.DB{tx}.TableExists("local_host_configs")) {
		_, err = tx.Exec(`
                          CREATE TABLE local_host_configs(
                                 url TEXT,
                                 user_defined_name TEXT,
                                 product_type VARCHAR(25)
                          );
                          
                          INSERT INTO local_host_configs VALUES('', 'engine_room', 'engine_room');`)
	}

	return err

}

func addLocalHostComponents(tx *sql.Tx) (err error) {
	if !(sqlite.DB{tx}.TableExists("local_host_compnonents")) {
		_, err = tx.Exec(`
                          CREATE TABLE local_host_components(
                                 user_defined_name TEXT,
                                 product_type VARCHAR(25)
                          );`)

	}

	return err
}
